package POEx::ProxySession;
BEGIN {
  $POEx::ProxySession::VERSION = '1.102750';
}

#ABSTRACT: Simple, IKC-like remote session proxifier

1;


=pod

=head1 NAME

POEx::ProxySession - Simple, IKC-like remote session proxifier

=head1 VERSION

version 1.102750

=head1 AUTHOR

Nicholas Perez <nperez@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2010 by Nicholas Perez.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut


__END__
